# encoding: utf-8

require 'fileutils'
require 'logger'
require 'tempfile'
require 'shellwords'
require 'ostruct'

unless Object.const_defined?(:ActiveSupport)
  require 'flow/core_ext/hash'
  require 'flow/core_ext/blank'
end

require 'flow/build/configuration'
require 'flow/build/Service/git'
require 'flow/build/service/parser'
require 'flow/build/version'

module Flow
  module Build
    class << self

      CONFIG = Configuration.new

      def configure(opts={})
        CONFIG.merge(opts)

        setup_general_logger!
        @configured = true
      end

      def configured?
        !!@configured
      end

      def clear!
        CONFIG.clear

        @logger     = nil
        @configured = false
      end

      def logger=(logger)
        @logger = logger
      end

      def logger
        @logger
      end

      def build_script(data)
        Flow::Build::Service::Parser.new(data).script
      end

      private

      def setup_general_logger!
        if [:info, :debug, :error, :warn].all?{ |meth| CONFIG[:log].respond_to?(meth) }
          @logger = CONFIG[:log]
        else
          @logger = Logger.new(CONFIG[:log])
          @logger.level = CONFIG[:log_level]
        end
      end
    end
  end
end
