# encoding: utf-8

require_relative './env'
require_relative './git'
require_relative './parser/ruby'
require_relative './parser/nodejs'
require_relative './parser/android'
require_relative './parser/objective_c'

module Flow
  module Build
    module Service
      class Parser
        include Ruby
        include Nodejs
        include Android
        include ObjectiveC

        attr_reader :object
        attr_accessor :scripts

        # flow-api Project object
        def initialize(object)
          @object  = object
          @scripts = {}

          apply
        end

        # options = { echo: false, time: false, retry: false, time: false }
        def flow_cmd(sh, options)
          sh = "flow_cmd " + sh.shellescape
          options.each { |k, v| sh += " --#{k}" if v }
          sh
        end

        def apply
          object.versions.each_with_index do |version, index|
            scripts[index] = [
              header,
              set_env,
              generate_git_script,
              generate_language_script(version),
              footer
            ].join("\n")
          end
        end

        def header
          File.read(File.expand_path('../parser/templates/header.sh', __FILE__))
        end

        def footer
          File.read(File.expand_path('../parser/templates/footer.sh', __FILE__))
        end

        def set_env
          Env.new(object.envs).apply
        end

        def generate_git_script
          Git.new(
            git_url:   object.git_url,
            ssh_key:   object.private_key,
            repo_dir:  object.repo_dir,
            repo_name: object.repo_name,
            branch:    object.branch
          ).apply
        end

        def generate_language_script(version)
          case object.language
          when 'ruby'
            ruby_apply(version)
          when 'android'
            # Android
          when 'objective_c'
            # Ios
          when 'nodejs'
            nodejs_apply(version)
            # Nodejs
          end
        end
      end
    end
  end
end
